/*
 * $Id: statusbar.prg 396 2010-10-15 07:49:43Z vouchcac $
 */

/*
 * HMG Source Code
 *
 * Copyright 2002-2010 Roberto Lopez <mail.box.hmg@gmail.com>
 * http://sites.google.com/site/hmgweb/
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or( at your option ) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License FOR more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this software; see the file COPYING. IF not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA( or
 * visit the web site http://www.gnu.org/ ).
 *
 * As a special exception, you have permission FOR additional uses of the text
 * contained in this release of HMG.
 *
 * The exception is that, IF you link the HMG library with other
 * files to produce an executable, this does not by itself cause the resulting
 * executable to be covered by the GNU General Public License.
 * Your use of that executable is in no way restricted on account of linking the
 * HMG library code into it.
 */

#include "hbclass.ch"
#include "common.ch"

#define QIcon_Normal                              0    // Display the pixmap when the user is not interacting with the icon, but the functionality represented by the icon is available.
#define QIcon_Disabled                            1    // Display the pixmap when the functionality represented by the icon is not available.
#define QIcon_Active                              2    // Display the pixmap when the functionality represented by the icon is available and the user is interacting with the icon, FOR example, moving the mouse over it or clicking it.
#define QIcon_Selected                            3    // Display the pixmap when the item represented by the icon is selected.

//enum #define QIcon_State
//This enum describes the state FOR which a pixmap is intended to be used. The state can be:
//
#define QIcon_On                                  0    // Display the pixmap when the widget is in an "on" state
#define QIcon_Off                                 1    // Display the pixmap when the widget is in an "off" state

#define QFrame_Plain                              0x0010   // the frame and contents appear level with the surroundings; draws using the palette QPalette::WindowText color( without any 3D effect )
#define QFrame_Raised                             0x0020   // the frame and contents appear raised; draws a 3D raised line using the light and dark colors of the current color group
#define QFrame_Sunken                             0x0030   // the frame and contents appear sunken; draws a 3D sunken line using the light and dark colors of the current color group
// Frame Shape
#define QFrame_NoFrame                            0        // QFrame draws nothing
#define QFrame_Box                                0x0001   // QFrame draws a box around its contents
#define QFrame_Panel                              0x0002   // QFrame draws a panel to make the contents appear raised or sunken
#define QFrame_StyledPanel                        0x0006   // draws a rectangular panel with a look that depends on the current GUI style. It can be raised or sunken.
#define QFrame_HLine                              0x0004   // QFrame draws a horizontal line that frames nothing( useful as separator )
#define QFrame_VLine                              0x0005   // QFrame draws a vertical line that frames nothing( useful as separator )
#define QFrame_WinPanel                           0x0003   // draws a rectangular panel that can be

#define QEvent_MouseButtonRelease                 3        // Mouse release( QMouseEvent ).


/*----------------------------------------------------------------------*/

CLASS STATUSBAR FROM WINDOW

   // Internal Data
   DATA lCreated                                  INIT   .f.
   DATA oQTObject                                 INIT   NIL
   DATA aItemObjects                              INIT   {}
   DATA cFontName                                 INIT   NIL
   DATA nFontSize                                 INIT   NIL
   DATA lTop                                      INIT   .f.
   DATA oParent                                   INIT   NIL
   DATA oDateTimer                                INIT   NIL
   DATA oClockTimer                               INIT   NIL
   DATA oKeyBoardTimer                            INIT   NIL
   DATA aUserDefined                              INIT   {}   //L3W

   // Properties
   METHOD FontName                                SETGET
   METHOD FontSize                                SETGET
   METHOD Top                                     SETGET
   METHOD Item                                    SETGET

   // Methods
   METHOD New
   METHOD StatusItemAdd
   METHOD StatusBarTimersAdd
   METHOD StatusBarKeyBoardAdd
   METHOD StatusBarDateUpdate
   METHOD StatusBarClockUpdate
   METHOD StatusBarKeyBoardUpdate

ENDCLASS

/*----------------------------------------------------------------------*/

* METHOD Item( nIndex, cItemCaption ) CLASS STATUSBAR            //L3W
METHOD Item( nIndex, cItemCaption, cIconName ) CLASS STATUSBAR  //L3W added thrd parameter
   LOCAL oItem := NIL

   IF PCOUNT() == 3                                    //L3W
      IF nIndex > 0                                    //L3W
       ::aUserDefined[nIndex,2] := cIconName           //L3W
      ENDIF                                            //L3W
   ENDIF                                               //L3W
   IF PCOUNT() >= 2                                    //L3W
      IF nIndex > 0                                    //L3W
       ::aUserDefined[nIndex,1] := cItemCaption        //L3W
      ENDIF                                            //L3W
   ENDIF                                               //L3W
*   IF PCount() == 2                                   //L3W
   IF PCOUNT() >= 2                                    //L3W
      IF nIndex > 0                                    //L3W
         oItem := ::aItemObjects[nIndex]               //L3W
*         oItem:SetText( cItemCaption )                //L3W
* icon is included...                                  //L3W
         IF VALTYPE( ::aUserDefined[nIndex,2] ) != 'U' //L3W
            oItem:setText( '<img src="' + ::aUserDefined[nIndex,2] + '"> ' + ::aUserDefined[nIndex,1] )   //L3W
         ELSE                                          //L3W
            oItem:setText( ::aUserDefined[nIndex,1] )  //L3W
         ENDIF                                         //L3W
      ENDIF
   ELSE
      IF PCount() == 1
         IF nIndex > 0
            oItem := ::aItemObjects[nIndex]
            RETURN oItem:Text()
         ENDIF
      ENDIF
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD FontName( cFontName ) CLASS STATUSBAR

   IF PCount() == 0
      RETURN ::cFontName
   ELSE
      ::cFontName := cFontName
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD FontSize( nFontSize ) CLASS STATUSBAR

   IF PCount() == 0
      RETURN ::nFontSize
   ELSE
      ::nFontSize := nFontSize
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD Top( lTop ) CLASS STATUSBAR

   HB_SYMBOL_UNUSED( lTop )

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD StatusItemAdd( cItemCaption,nWidth,bBlock,cIconName,lFlat,lRaised,cToolTipText ) CLASS STATUSBAR
   LOCAL oFont, oWidget := QLabel()

   HB_SYMBOL_UNUSED( lFlat )

   IF ValType( nWidth ) != 'U'
      oWidget:resize( nWidth , oWidget:Height )
   ENDIF

   // IF icon is included...
   IF ValType( cIconName ) != 'U'
      oWidget:setText( '<img src="'+cIconName+'"> '+cItemCaption )
   ELSE
      oWidget:setText( cItemCaption )
   ENDIF
   IF lRaised
       oWidget:setFrameStyle( QFrame_StyledPanel+QFrame_Raised )
   ENDIF

   // IF any action needed
   IF valtype( bBlock ) == 'B'
      oWidget:connect( QEvent_MouseButtonRelease,bBlock )
   ENDIF

   // font specified or use default font?
   IF ValType( ::cFontName ) != 'U'
      oFont := QFont()
      oFont:SetFamily( ::cFontName )
      oWidget:setFont( oFont )
   ENDIF

   IF ValType( ::nFontSize ) != 'U'
      oFont := QFont()
      oFont:SetPointSize( ::nFontSize )
      oWidget:setFont( oFont )
   ENDIF

   // set tooltip text
   IF ValType( cToolTipText ) != 'U'
      oWidget:setToolTip( cToolTipText )
   ENDIF

   // store this label object alive internally!
   aadd( ::aItemObjects,oWidget )

   // store user defined value: text and image          //L3W
   AADD( ::aUserDefined, { cItemCaption, cIconName } )  //L3W

   // Finally add to statusbar
   ::oQTObject:addPermanentWidget( oWidget,1 )

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD StatusBarTimersAdd( nWidth,bBlock,lFlat,lRaised,cToolTipText,cMode ) CLASS STATUSBAR
   LOCAL oFont, oWidget

   HB_SYMBOL_UNUSED( lFlat )

   oWidget := QLabel()

   IF ValType( nWidth ) != 'U'
      oWidget:resize( nWidth , oWidget:Height )
   ENDIF

   // IF icon is included...
   IF lRaised
      oWidget:setFrameStyle( QFrame_StyledPanel+QFrame_Raised )
   ENDIF

   // IF any action needed
   IF valtype( bBlock ) == 'B'
      oWidget:connect( QEvent_MouseButtonRelease,bBlock )
   ENDIF

   // font specified or use default font?
   IF ValType( ::cFontName ) != 'U'
      oFont := QFont()
      oFont:SetFamily( ::cFontName )
      oWidget:setFont( oFont )
   ENDIF

   IF ValType( ::nFontSize ) != 'U'
      oFont := QFont()
      oFont:SetPointSize( ::nFontSize )
      oWidget:setFont( oFont )
   ENDIF

   // set tooltip text
   IF ValType( cToolTipText ) != 'U'
      oWidget:setToolTip( cToolTipText )
   ENDIF

   // store this label object alive internally!
   aadd( ::aItemObjects,oWidget )

   // Set the timer
   DO CASE
      CASE cMode == "D"  && Date mode
         ::oDateTimer := QTimer()
         ::oDateTimer:connect( "timeout()",{||Self:StatusBarDateUpdate( oWidget )} )
         ::oDateTimer:start( 1000 )
      CASE cMode == "C"  && Clock mode
         ::oClockTimer := QTimer()
         ::oClockTimer:connect( "timeout()",{||Self:StatusBarClockUpdate( oWidget )} )
         ::oClockTimer:start( 1000 )
   ENDCASE

   // Finally add to statusbar
   ::oQTObject:addPermanentWidget( oWidget,1 )

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD StatusBarKeyBoardAdd( nWidth,bBlock,lFlat,lRaised,cToolTipText ) CLASS STATUSBAR
   LOCAL oFont, oWidget

   HB_SYMBOL_UNUSED( lFlat )

   oWidget := QLabel()

   IF ValType( nWidth ) != 'U'
      oWidget:resize( nWidth , oWidget:Height )
   ENDIF

   // IF icon is included...
   IF lRaised
       oWidget:setFrameStyle( QFrame_StyledPanel+QFrame_Raised )
   ENDIF

   // IF any action needed
   IF valtype( bBlock ) == 'B'
      oWidget:connect( QEvent_MouseButtonRelease,bBlock )
   ENDIF

   // font specified or use default font?
   IF ValType( ::cFontName ) != 'U'
      oFont := QFont()
      oFont:SetFamily( ::cFontName )
      oWidget:setFont( oFont )
   ENDIF
   IF ValType( ::nFontSize ) != 'U'
      oFont := QFont()
      oFont:SetPointSize( ::nFontSize )
      oWidget:setFont( oFont )
   ENDIF

   // set tooltip text
   IF ValType( cToolTipText ) != 'U'
      oWidget:setToolTip( cToolTipText )
   ENDIF

   // store this label object alive internally!
   aadd( ::aItemObjects,oWidget )

   // Set the timer to update the keyboard status frequently
   ::oKeyBoardTimer := QTimer()
   ::oKeyBoardTimer:connect( "timeout()",{||Self:StatusBarKeyBoardUpdate( oWidget )} )
   ::oKeyBoardTimer:start( 1000 )

   // Finally, add to statusbar
   ::oQTObject:addPermanentWidget( oWidget,1 )

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD StatusBarDateUpdate( oWidget ) CLASS STATUSBAR
   LOCAL dDate := date()
   oWidget:setText( dtoc( dDate ) )
   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD StatusBarClockUpdate( oWidget ) CLASS STATUSBAR
   LOCAL cTime := time()
   oWidget:setText( cTime )
   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD StatusBarKeyBoardUpdate( oWidget ) CLASS STATUSBAR
   LOCAL lNum:= KSETNUM()
   LOCAL lCaps:= KSETCAPS()
   LOCAL lIns:= KSETINS()

   oWidget:setText( '<img src="'+iif( lNum,"led_on.ico","led_off.ico" )+'">'+;
                   '<img src="'+iif( lCaps,"led_on.ico","led_off.ico" )+'">'+;
                   '<img src="'+iif( lIns,"led_on.ico","led_off.ico" )+'">';
    )

   RETURN NIL

/*----------------------------------------------------------------------*/
//                             Methods
/*----------------------------------------------------------------------*/

METHOD New( oParent ) CLASS STATUSBAR

   IF valtype( oParent ) == 'U'
      oParent := ::oDefaultParent
   ENDIF

   oParent:oStatusBar := Self
   ::oQTObject := QStatusBar()

   RETURN Self

/*----------------------------------------------------------------------*/
